@@ -71,6 +71,9 @@ class Resource (sb.Base, sa.Attributes, async.Async) :
     the agreed time duration has passed -- this is represented by the `EXPIRED`
     state.
     """
+    # FIXME: 
+    #   - we don't use PENDING like this, yet
+    #   - include state diagram (also for jobs btw)
 
     # --------------------------------------------------------------------------
     #
@@ -144,8 +147,10 @@ class Resource (sb.Base, sa.Attributes, async.Async) :
         self._attributes_set_getter (const.DESCRIPTION , self.get_description )
 
 
-        # we need the ID to be an URL, as we don't have a scheme otherwise,
-        # which means we can't select an adaptor.  Duh! :-/
+        # FIXME: we need the ID to be or to include an URL, as we don't have
+        # a scheme otherwise, which means we can't select an adaptor.  Duh! :-/
+        
+        # FIXME: documentation for attributes is missing.
 
         # param checks
         scheme = None
@@ -161,7 +166,6 @@ class Resource (sb.Base, sa.Attributes, async.Async) :
 
 
         if not session :
-
             session = ss.Session (default=True)
 
         self._base = super  (Resource, self)
@@ -199,13 +203,14 @@ class Resource (sb.Base, sa.Attributes, async.Async) :
         a specific set of attributes.  At some point in time, while the
         resource is running, the application requirements on the resource may
         have changed -- in that case, the application can request to change the
-        resource attributes on the fly.
+        resource's configuration on the fly.
 
         This method cannot be used to change the type of the resource.  Backends
         may or may not support this operation -- if not,
-        a :class:`saga.NotImplemented` exception is raised.  If yes, then the
-        semantics of the method is equivalent to the semantics of the
-        :func:`acquire` call on the :class:`saga.resource.Manager` class.
+        a :class:`saga.NotImplemented` exception is raised.  If the method is
+        supported, , then the semantics of the method is equivalent to the
+        semantics of the :func:`acquire` call on the
+        :class:`saga.resource.Manager` class.
         """
 
         return self._adaptor.reconfig (descr, ttype=ttype)
@@ -242,11 +247,21 @@ class Resource (sb.Base, sa.Attributes, async.Async) :
         :type  state: float
         :param state: time to block while waiting.
 
-        The semantics of this method is equivalent to the semantics of the
-        :func:`wait` call on the :class:`saga.resource.Manager` class.
+        This method will block until the resource entered the specified state,
+        or until `timeout` seconds have passed -- whichever occurs earlier.  If
+        the resource is in a final state, the call will raise and
+        :class:`saga.IncorrectState` exception when asked to wait for any
+        non-final state.
+
+        A negative `timeout` value represents an indefinit timeout.
         """
-        # FIXME: should we also have a wait() on the manager?
-        # FIXME: right now, we can not put a resource in a task container
+
+        # FIXME: 
+        #   - right now, we can not put a resource in a `TaskContainer`, because
+        #     it is not a `Task`.  We need to either reconsider the class
+        #     hierarchy, and then inherit a `ResourceContainer` from
+        #     `TaskContainer` (see job); or we implement a `ResourceContainer`
+        #     from scratch...
     
         return self._adaptor.wait (state, timeout, ttype=ttype)
 
@@ -255,7 +270,8 @@ class Resource (sb.Base, sa.Attributes, async.Async) :
     #
     @sus.takes   ('Resource', sus.optional (sus.one_of (SYNC, ASYNC, TASK)))
     @sus.returns ((sus.nothing, basestring, st.Task))
-    def get_id   (self, ttype=None) : return self._adaptor.get_id            (ttype=ttype)
+    def get_id   (self, ttype=None) : 
+        return self._adaptor.get_id            (ttype=ttype)
 
 
     # --------------------------------------------------------------------------
@@ -264,7 +280,8 @@ class Resource (sb.Base, sa.Attributes, async.Async) :
     @sus.returns  ((sus.one_of (const.COMPUTE, 
                                 const.STORAGE, 
                                 const.NETWORK), st.Task))
-    def get_rtype (self, ttype=None) : return self._adaptor.get_rtype         (ttype=ttype)
+    def get_rtype (self, ttype=None) : 
+        return self._adaptor.get_rtype         (ttype=ttype)
 
 
     # --------------------------------------------------------------------------
@@ -279,46 +296,53 @@ class Resource (sb.Base, sa.Attributes, async.Async) :
                                 const.DONE    ,
                                 const.FAILED  ,
                                 const.FINAL   ), st.Task))
-    def get_state (self, ttype=None) : return self._adaptor.get_state         (ttype=ttype)
+    def get_state (self, ttype=None) : 
+        return self._adaptor.get_state         (ttype=ttype)
 
 
     # --------------------------------------------------------------------------
     #
     @sus.takes   ('Resource', sus.optional (sus.one_of (SYNC, ASYNC, TASK)))
     @sus.returns ((sus.nothing, basestring, st.Task))
-    def get_state_detail (self, ttype=None) : return self._adaptor.get_state_detail  (ttype=ttype)
+    def get_state_detail (self, ttype=None) : 
+        return self._adaptor.get_state_detail  (ttype=ttype)
 
 
     # --------------------------------------------------------------------------
     #
     @sus.takes     ('Resource', sus.optional (sus.one_of (SYNC, ASYNC, TASK)))
     @sus.returns   ((sus.nothing, basestring, st.Task))
-    def get_access (self, ttype=None) : return self._adaptor.get_access        (ttype=ttype)
+    def get_access (self, ttype=None) : 
+        return self._adaptor.get_access        (ttype=ttype)
 
 
     # --------------------------------------------------------------------------
     #
     @sus.takes      ('Resource', sus.optional (sus.one_of (SYNC, ASYNC, TASK)))
     @sus.returns    ((basestring, st.Task))
-    def get_manager (self, ttype=None) : return self._adaptor.get_manager       (ttype=ttype)
+    def get_manager (self, ttype=None) : 
+        return self._adaptor.get_manager       (ttype=ttype)
 
 
     # --------------------------------------------------------------------------
     #
     @sus.takes   ('Resource', sus.optional (sus.one_of (SYNC, ASYNC, TASK)))
     @sus.returns ((sus.nothing, descr.Description, st.Task))
-    def get_description  (self, ttype=None) : return self._adaptor.get_description   (ttype=ttype)
+    def get_description  (self, ttype=None) : 
+        return self._adaptor.get_description   (ttype=ttype)
 # 
 # ------------------------------------------------------------------------------
 
 
+
 # ------------------------------------------------------------------------------
 #
 class Compute (Resource) :
     """
-    A Compute resource is a resource which can execute compute jobs.  As such,
-    the 'Access' attribute of the compute resource (a URL) can be used to create
-    a :class:`saga.job.Service` instance to submit jobs to.
+    A Compute resource is a resource which provides compute capabilities, i.e.
+    which can execute compute jobs.  As such, the 'Access' attribute of the
+    compute resource (a URL) can be used to create a :class:`saga.job.Service`
+    instance to submit jobs to.
     """
 
     # --------------------------------------------------------------------------
@@ -347,8 +371,9 @@ class Compute (Resource) :
 #
 class Storage (Resource) :
     """
-    A Storage resource is a resource which can store data.  As such,
-    the 'Access' attribute of the storage resource (a URL) can be used to create
+    A Storage resource is a resource which has storage capabilities, i.e. the
+    ability to persistently store, organize and retrieve data.  As such, the
+    'Access' attribute of the storage resource (a URL) can be used to create
     a :class:`saga.filesystem.Directory` instance to manage the resource's data
     space.
     """
@@ -377,7 +402,7 @@ class Storage (Resource) :
 #
 class Network (Resource) :
     """
-    A Network resource is a resource which can connect arbitrary resources.
+    A Network resource is a resource which has network capabilities.
     """
 
     # --------------------------------------------------------------------------
