@@ -67,7 +67,6 @@ class PTYShell (object) :
     details.
 
     Usage Example::
-    ^^^^^^^^^^^^^^^
 
         # start the shell, find its prompt.  
         self.shell = saga.utils.pty_shell.PTYShell ("ssh://user@remote.host.net/", contexts, self._logger)
@@ -93,8 +92,8 @@ class PTYShell (object) :
         assert (len(pbs_job_script) == int(out))
 
 
-    Data Staging and Data Management:
-    ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+    **Data Staging and Data Management:**
+    
 
     The PTYShell class does not only support command execution, but also basic
     data management: for SSH based shells, it will create a tunneled scp/sftp
@@ -108,8 +107,7 @@ class PTYShell (object) :
     management operations.  
 
 
-    Asynchronous Notifications:
-    ^^^^^^^^^^^^^^^^^^^^^^^^^^^
+    **Asynchronous Notifications:**
 
     A third pty process will be created for asynchronous notifications.  For
     that purpose, the shell started on the first channel will create a named
@@ -155,8 +153,7 @@ class PTYShell (object) :
     usually 4096), or to lock the pipe on larger writes.
 
 
-    Automated Restart, Timeouts:
-    ^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+    **Automated Restart, Timeouts:**
 
     For timeout and restart semantics, please see the documentation to the
     underlying :class:`saga.utils.pty_process.PTYProcess` class.
@@ -240,6 +237,10 @@ class PTYShell (object) :
             self.logger.debug    ("running command shell: %s" % command_shell)
             self.pty_shell.write ("stty -echo ; %s\n"         % command_shell)
 
+            # make sure this worked, and that we find the prompt. We use
+            # a versatile prompt pattern to account for the custom shell case.
+            self.find (["^(.*[\$#%>])\s*$"])
+
             # make sure this worked, and that we find the prompt. We use
             # a versatile prompt pattern to account for the custom shell case.
             try :
@@ -247,14 +248,11 @@ class PTYShell (object) :
                 self.run_async  ("unset PROMPT_COMMAND ; "
                                      + "PS1='PROMPT-$?->'; "
                                      + "PS2=''; "
-                                     + "export PS1 PS2 2>&1 >/dev/null; true\n")
+                                     + "export PS1 PS2 2>&1 >/dev/null\n")
                 self.set_prompt (new_prompt="PROMPT-(\d+)->$")
 
                 self.logger.debug ("got new shell prompt")
 
-            except se.SagaException as e :
-                raise
-
             except Exception as e :
                 raise se.NoSuccess ("Shell startup on target host failed: %s" % e)
 
@@ -310,10 +308,12 @@ class PTYShell (object) :
             try :
 
                 match = None
+                fret  = None
 
-                while not match :
-                    _, match = self.pty_shell.find ([self.prompt], _PTY_TIMEOUT)
-
+                while fret == None :
+                    fret, match = self.pty_shell.find ([self.prompt], _PTY_TIMEOUT)
+                
+              # self.logger.debug  ("find prompt '%s' in '%s'" % (self.prompt, match))
                 ret, txt = self._eval_prompt (match)
 
                 return (ret, txt)
@@ -406,11 +406,11 @@ class PTYShell (object) :
 
                     # FIXME: how do we know that _PTY_TIMOUT suffices?  In particular if
                     # we actually need to flush...
-                    _, match  = self.pty_shell.find ([self.prompt], delay)
+                    fret, match = self.pty_shell.find ([self.prompt], delay)
 
                   # self.logger.error  ("got match (%s)" % match)
 
-                    if not match :
+                    if  fret == None :
                     
                         retries += 1
                         if  retries > 10 :
@@ -446,9 +446,9 @@ class PTYShell (object) :
                 self.run_async ('printf "SYNCHRONIZE_PROMPT\n"')
 
                 # FIXME: better timout value?
-                _, match = self.pty_shell.find (["SYNCHRONIZE_PROMPT"], timeout=1.0)  
+                fret, match = self.pty_shell.find (["SYNCHRONIZE_PROMPT"], timeout=1.0)  
 
-                if not match :
+                if  fret == None :
                     # not find prompt after blocking?  BAD!  Restart the shell
                     self.finalize (kill_pty=True)
                     raise se.NoSuccess ("Could not synchronize prompt detection")
@@ -605,9 +605,9 @@ class PTYShell (object) :
                     prompt = new_prompt
 
                 # command has been started - now find prompt again.  
-                _, match = self.pty_shell.find ([prompt], timeout=-1.0)  # blocks
+                fret, match = self.pty_shell.find ([prompt], timeout=-1.0)  # blocks
 
-                if not match :
+                if  fret == None :
                     # not find prompt after blocking?  BAD!  Restart the shell
                     self.finalize (kill_pty=True)
                     raise se.IncorrectState ("run_sync failed, no prompt (%s)" % command)
@@ -628,9 +628,9 @@ class PTYShell (object) :
                     stdout =  txt
 
                     self.pty_shell.write ("cat %s\n" % _err)
-                    _, match = self.pty_shell.find ([self.prompt], timeout=-1.0)  # blocks
+                    fret, match = self.pty_shell.find ([self.prompt], timeout=-1.0)  # blocks
 
-                    if not match :
+                    if  fret == None :
                         # not find prompt after blocking?  BAD!  Restart the shell
                         self.finalize (kill_pty=True)
                         raise se.IncorrectState ("run_sync failed, no prompt (%s)" \
